<?php

/**
 * @file
 * controller for the frontend search
 */

/**
 * menu callback for archibald content overview
 *
 * @return string html
 */
function archibald_content_overview() {
  global $language;

  $reset_form = FALSE;

  if (!user_access('view archibald content')) {
    $nid = variable_get('archibald_alternative_homepage_node', '');
    $node = '';
    if (!empty($nid)) {
      $node = node_load($nid);
    }
    if (!empty($node)) {
      $html = node_show($node);
    }
    else {
      $html = t('Welcome to our website');
    }
  }
  else {
    drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');
    drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald.css');

    // for link search like vocabular items
    if (empty($_POST) && isset($_GET['query'])) {
      $_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY] = $_GET['query'];
    }

    $num_per_page = variable_get('archibald_content_num_per_page', 25);

    $query = '';
    if (isset($_POST['query'])) {
      $_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY] = filter_xss($_POST['query']);
    }
    elseif (isset($_GET['query'])) {
      $_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY] = filter_xss($_GET['query']);
    }

    if (isset($_POST['op']) && $_POST['op'] == t('Reset')) {
      $reset_form = TRUE;
      $_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY] = '';
    }

    if (!empty($_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY])) {
      $query = $_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY];
    }

    if (variable_get('archibald_solr_search_activ', 0) == 1 && module_exists('apachesolr')) {
      $search = new ArchibaldLomSolrSearch();
      $search->init(FALSE);
      drupal_add_js(
        drupal_get_path('module', 'archibald') .
        '/js/client_overview_facets.js'
      );

      $search->setLimit($num_per_page);
      $search->setStart(pager_find_page() * $num_per_page);

      $search->enableFacets();

      if (!empty($query)) {
        $in_query_filters = array(
          'learning_resourcetype' => 'sm_learning_resourcetype',
          'intendet_enduserrole' => 'sm_intendet_enduserrole',
          'difficulty' => 'ss_difficulty',
          'education_context' => 'sm_context',
          'keywords' => 'tm_keywords',
          'coverage' => 'tm_coverage',
          'author' => 'sm_author',
          'editor' => 'tm_contributor',
          'publisher' => 'tm_contributor',
        );
        $query = strtr($query, $in_query_filters);

        $search->setQuery($query);
      }

      if (variable_get('archibald_search_in_current_language', 1) == TRUE) {
        $search->setFilter('ss_language', $language->language);
      }

      // field to look for search keywords with boost options
      $qf_values = array(
        'label',
        'content',
        'tm_keywords',
        'tm_coverage',
        'ts_otherPlattformRequirements',
        'ts_education_description',
        'tm_contributor',
        'tm_vocabular_text',
      );

      $qf_base = '';
      foreach ($qf_values as $key) {
        $value = archibald_get_solr_bias_value($key);
        if (!empty($value)) {
          $qf_base .= $key . '^' . $value . ' ';
        }
      }
      $qf_base = trim($qf_base);

      $search->setQueryFields($qf_base);

      // get facet checkboxes and process them,
      // this we needs to do before ->search() and ->get_filter_options()
      foreach ($_POST as $key => $val) {
        if ( !$reset_form && drupal_substr($key, 0, 6) == 'facet_') {
          $search->setFacetFilter(
            drupal_substr($key, 6),
            array_keys($val)
          );
        }
      }

      // echo '<PRE>';
      // $q = $search->search(true);
      // print_r($q->getParams());
      // print_r($q->getSolrParams());
      // print_r($q->getSolrSort());
      // echo '</PRE>';
      // exit;

      $search_result = $search->search(TRUE);
      pager_default_initialize($search->getTotal(), $num_per_page);

      $search_form = drupal_get_form(
        'archibald_content_overview_form',
        $search->getFacetOptions(),
        $search->getTotal()
      );

      $html = theme(
        'archibald_content_overview',
        array(
          'search_result' => $search_result,
          'search_form' => $search_form,
          'search_result_count' => $search->getTotal(),
        )
      );
    }
    else {
      // use the default sql search

      $search = new ArchibaldLomSQLSearch();

      if (variable_get('archibald_search_in_current_language', 1) == TRUE) {
        $search_in_lang = $language->language;
        $search->setFilterLanguage($search_in_lang);
      }
      else {
        $search_in_lang = 'any';
      }

      $search->setLimit($num_per_page);
      $search->setQuery(
        $query,
        $search_in_lang
      );
      $search->setLocalPublishedFilter(TRUE);
      $search->getLocalPublishedVersion();

      $total_record_count = $search->getTotalRecordCount();

      $page = pager_default_initialize($total_record_count, $num_per_page);
      $search->setOffset($page * $num_per_page);

      $search_form = drupal_get_form('archibald_content_overview_form');

      $html = theme(
        'archibald_content_overview',
        array(
          'search_result' => $search->search(TRUE),
          'search_form' => $search_form,
          'search_result_count' => $total_record_count,
        )
      );
    }
  }

  if (!empty($_POST['ajax'])) {
    echo $html;
    die();
  }

  return $html;
}

/**
 * search formular for archibald resources
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function archibald_content_overview_form($form, &$form_state, $filter_options = array(), $search_results = 0) {

  $query = '';
  if (!empty($_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY])) {
    $query = $_SESSION[ARCHIBALD_FRONT_END_SESSION_SEARCH_KEY];
  }

  $form['query'] = array(
    '#type' => 'textfield',
    '#title' => t('Term'),
    '#default_value' => $query,
    '#size' => 40,
  );

  if ($search_results > 0 && !empty($filter_options)) {
    $weight = 5;
    foreach ($filter_options as $filter_option_key => $filter_option) {
      // Count the number of results for this option
      $count = archibald_content_overview_form_countfacets($filter_option);
      // Show the accordion only if the facet filter contains an option with results
      if ($count) {
        $key = 'facet_' . $filter_option_key;
        $form[$key] = array(
          '#type' => 'fieldset',
          '#title' => t('Filter by') . ' ' .  t($filter_option_key),
          '#weight' => $weight,
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          '#tree' => TRUE,
        );
        archibald_content_overview_form_facets2form_recursiv(
          $form[$key],
          $filter_option,
          $key
        );
        $weight++;
      }
    }
  }

  $form['search'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
    '#weight' => 9999,
  );
  $form['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset'),
    '#weight' => 9999,
  );
  $form['#theme'] = 'archibald_content_overview_form';

  return $form;
}

/**
 * process facets recursively and return count of results in an option
 *
 * @param array $options
 * @param int $count
 *
 */
function archibald_content_overview_form_countfacets($options, $count = 0) {
  foreach ($options as $option) {
    $count += $option->count;
    if (count($option->vals)) $count = archibald_content_overview_form_countfacets($option->vals, $count);
  }
  return( $count );
}

/**
 * process facets recursively and display them
 *
 * @param array $form
 * @param array $items
 * @param string $parent_key
 *   Populate the form elements
 */
function archibald_content_overview_form_facets2form_recursiv(&$form, $items, $parent_key) {

  foreach ($items as $item) {
    $form[$item->key] = array(
      '#type' => 'checkbox',
      '#title' => $item->name,
      '#default_value' => $item->activ,
      '#result_count' => $item->count,
      '#attributes' => array(
        'key' => $item->key,
        'class' => array('facet_id_' . $item->key),
      ),
      '#tree' => TRUE,
      '#parents' => array($parent_key, $item->key),
    );

    archibald_content_overview_form_facets2form_recursiv(
      $form[$item->key],
      $item->vals,
      $parent_key
    );
  }
}

/**
 * theme the search form
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_content_overview_form($form) {
  $form = reset($form);
  unset($form['query']['#title']);
  $html = '';
  $html .= '<div>';
  $html .= theme('textfield', $form['query']) . drupal_render($form['search']) . drupal_render($form['reset']);
  $html .= '</div>';

  foreach (element_children($form) as $key) {
    if (drupal_substr($key, 0, 6) == 'facet_') {
      $html .= '<div class="details">';
      $html .= ' <p><a href="JavaScript:void(0);" id="' . $key . '" class="btndetails inactive remember_choice sorl_search_facet">' . '<span>' . t($form[$key]['#title']) . '</span></a></p>';
      $html .= ' <div style="display: block;" class="detailscontainer recusrivcontainer">';
      $html .= '  <ul class="' . $key . '">';
      $html .= theme_archibald_content_overview_facet_form($form[$key]);
      $html .= '  </ul>';
      $html .= '  <div class="clear"></div>';
      $html .= ' </div>';
      $html .= '</div>';
    }
  }

  return $html;
}

/**
 * not an offical theme functions, it ill called direcktly.
 * Here we render the filter checkboxes
 *
 * @param array $form
 * @param integer $sum_count
 *
 * @return string html
 */
function theme_archibald_content_overview_facet_form($form, &$sum_count = 0) {
  $content = '';
  $found_enabled = FALSE;
  foreach (element_children($form) as $checkbox_key) {
    $found_enabled_child = FALSE;

    // this nee ned to find out if a child have any matches
    $child_count = 0;
    $tmp_content = theme_archibald_content_overview_facet_form(
      $form[$checkbox_key],
      $child_count
    );

    // a child is enabled but all have not counts
    if ($child_count == -1) {
      $found_enabled       = TRUE;
      $found_enabled_child = TRUE;
      $child_count         = 0;
    }

    if ($form[$checkbox_key]['#default_value'] == TRUE) {
      // current child is enabled
      $found_enabled = TRUE;
    }

    $result_count = (INT) $form[$checkbox_key]['#result_count'];
    $child_count += $result_count;
    $sum_count += $child_count;

    // if current has 0 relts but its child ,get the result sum of child
    // work around for lerning resurce type
    if (empty($result_count) && !empty($child_count)) {
      $result_count = $child_count;
      $form[$checkbox_key]['#disabled'] = TRUE;
    }

    // append count to title
    $form[$checkbox_key]['#title'] .= ' (' . $result_count . ')';
    $form[$checkbox_key]['#attributes']['class'][] = 'sorl_search_facet_select';

    /* display item if:
     *  item or child have facet hits
     * OR
     *  item or child is activ (checkbox was checked)
     */

    if (!empty($child_count) ||
      $form[$checkbox_key]['#default_value'] == TRUE ||
      $found_enabled_child == TRUE
    ) {
      $content .= '<li>';
      if (!empty($form[$checkbox_key]['#disabled'])) {
        // only show title not the text box if it is disabled
        $content .= $form[$checkbox_key]['#title'];
      }
      else {
        $content .= drupal_render($form[$checkbox_key]);
      }

      if (!empty($tmp_content)) {
        $content .= '<ul>' . $tmp_content . '</ul>';
      }
      $content .= '</li>';
    }
  }

  // if a child is enabled but all have no facet hits
  // set to -1 to notify parent instance to let this tree open
  if ($found_enabled == TRUE && $sum_count < 1) {
    $sum_count = -1;
  }

  return $content;
}

